;*****************************************************************************
;				NICST TOOL
;
;  RC-5 Part 1 test for TEB bands. Calculates the background subtracted DN and
;  standard deviations. To find the saturation, use only the fixed high gain collects. 
;  For the gain switching, use only the auto gain collects for M13.
;
;  Input:
;  1) File with inpath and outpath.
;
;  Output:
;  1) dn(collect,band,HAM,detector,subsample,3] -- dn for frame average then scan average,
;     scan average then frame average, and average over all scans and frames 
;     with a 3 sigma kick out at each step.
;  2) std(collect,band,HAM,detector,subsample,3] -- std for frame average then scan average,
;     scan average then frame average, and average over all scans and frames 
;     with a 3 sigma kick out at each step.
;  3) slope1(collect,band,HAM,detector,subsample,scan,2] -- coefficients from a straight line fit to
;     the dn versus frame.
;  4) slope2(collect,band,HAM,detector,subsample,2] -- coefficients from a straight line fit to
;     the dn versus scan.
;  5) dn_col(collect,band,HAM,detector,subsample,scan,3] -- dn for frame average with a 3 sigma kick 
;     out at each step for (0=EV, 1=SV, 2=OBC). The EV and OBC indices are SV subtracted.
;  6) std_col(collect,band,HAM,detector,subsample,scan,3] -- std for frame average with a 3 sigma kick 
;     out at each step for (0=EV, 1=SV, 2=OBC). The EV and OBC indices are SV subtracted.
;  7) dn_col_2(collect,band,HAM,detector,subsample,frames] -- dn for scan average with a 3 sigma kick 
;     out at each step for (0=EV, 1=OBC). The EV and OBC indices are SV subtracted.
;  8) std_col_2(collect,band,HAM,detector,subsample,frames] -- std for scan average with a 3 sigma kick 
;     out at each step for (0=EV, 1=OBC). The EV and OBC indices are SV subtracted.
;  
;  v1.0 - 07/07/2008 - Jeff McIntire
;*****************************************************************************

PRO RC5_Gain_dn_3, infile

;*****************************************************************************
;  Read in input file, sav TEB file, and get list of dump files.
;*****************************************************************************

nlines=File_Lines(infile)
ab=''
header=strarr(1)
inpath=strarr(1)		; Input path.
outpath=strarr(1)		; Output path.
rvs_in=strarr(1)		; RVS .sav file.
param_in=strarr(1)		; EDU coeff .sav file.
rsrpath=strarr(1)		; RSR .sav file.
fnumber=strarr(1)
header_size=intarr(1)

openr,lun,infile,/Get_Lun	
readf,lun,header	
readf,lun,inpath
readf,lun,outpath
readf,lun,rvs_in
readf,lun,param_in
readf,lun,rsrpath
readf,lun,fnumber
readf,lun,header_size
Free_Lun,lun

n_collect,inpath,ncollects,type,0

restore,'/home/jmcintire/VIIRS_Data/general.TEB.sav'

; Get list of .dmp files from specified path. 
spawn, 'ls -C1 ' + inpath + '*.dmp', list_files
IF N_ELEMENTS(list_files) EQ 0 THEN goto,skipout
nfile=N_ELEMENTS(list_files)

;*****************************************************************************
;  Some definitions.
;*****************************************************************************

max_scan=100					; Total # scans.

outfile=' '					; Output file name, which is reasigned.

; DN values per collect (0=EV, 1=SV).
dn_col=dblarr(ncollects,nbands_TEB,nham,maxdet,maxgain,maxsubsets,max_scan,2)
dn_col_2=dblarr(ncollects,nbands_TEB,nham,maxdet,maxgain,maxsubsets,200)

; Final dn (0=frame,scan 1=scan,frame 2frame+scan).
dn=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,maxgain,3)

; STD values per collect (0=EV, 1=SV).
std_col=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,maxgain,max_scan,2)
std_col_2=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,maxgain,200)

; Final std (0=frame,scan 1=scan,frame 2frame+scan).
std=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,maxgain,3)

;*****************************************************************************
;  Read in the dump files for SV.
;*****************************************************************************

m=0
a5=0
FOR ifile=0, nfile-1 DO BEGIN

    nlines=File_Lines(list_files[ifile])
    IF nlines EQ 0 THEN continue
    
    a2=strsplit(list_files[ifile],'/',/Extract)
    a3=N_ELEMENTS(a2)
    a4=strmid(a2[a3-1],strpos(a2[a3-1],'_',/Reverse_Search)+1,1) ; Determine I or M band.
    a8=strmid(a2[a3-1],strpos(a2[a3-1],'/',/Reverse_Search)+10,2) ; Determine EV or SV.
    
    ; Keep only the SV.
    IF a8 NE 'SV' THEN continue
    
    ; Determine the relative collect #.
    m=m+1
    IF m EQ 9 THEN BEGIN
        a5=a5+1
        m=1
    ENDIF
    print,list_files[ifile]
    
    read_dmpd2,list_files[ifile],header_size,uaid,collect,scan,ham,apid,band,dnb_s,det,dsect,frame,offset,gain_type,dns

    ; Translate the band name to a number.
    IF band[0] EQ 'I4' THEN b=0
    IF band[0] EQ 'I5' THEN b=1
    IF band[0] EQ 'M12' THEN b=2
    IF band[0] EQ 'M13' THEN b=3
    IF band[0] EQ 'M14' THEN b=4
    IF band[0] EQ 'M15' THEN b=5
    IF band[0] EQ 'M16A' THEN b=6
    IF band[0] EQ 'M16B' THEN b=7
    
    IF band_type_TEB[b] EQ 1 THEN max_frame=96 ; Total # frames (I band).
    IF band_type_TEB[b] EQ 0 THEN max_frame=48 ; Total # frames (M band).
    
;*****************************************************************************
;  Write DN values to an array for SV.
;*****************************************************************************

    dn_frame=dblarr(nham,maxdet,maxsubsets,maxgain,max_scan,max_frame)
    iscan=0
    idet=0
    iframe=0
    subset=0
    FOR i=0L, nlines-1 DO BEGIN
        IF scan[i] EQ iscan+1 AND det[i] EQ idet+1 THEN BEGIN
            dn_frame[ham[i],idet,subset,gain_type[i],iscan,iframe]=dns[i]
            iframe=iframe+1
            IF iframe EQ max_frame THEN BEGIN
                idet=idet+1
                iframe=0
                IF idet EQ detectors_TEB[b] THEN BEGIN
                    idet=0
                    iscan=iscan+1
                ENDIF
            ENDIF
        ENDIF
        IF a4 EQ 'I' THEN BEGIN
            IF subset EQ 0 THEN BEGIN
                subset=1
                goto,skip_over_1
            ENDIF
            IF subset EQ 1 THEN subset=0
            skip_over_1:
        ENDIF
    ENDFOR

;*****************************************************************************
;  Average over frames and calculate the standard deviation.
;*****************************************************************************

    FOR iham=0, nham-1 DO BEGIN
        FOR idet=0, detectors_TEB[b]-1 DO BEGIN
            FOR isubset=0, subsets_TEB[b]-1 DO BEGIN
                FOR igain=0, 1 DO BEGIN
                    FOR iscan=0, max_scan-1 DO BEGIN
                        dn_col[a5,b,iham,idet,isubset,igain,iscan,1]= $
                          sig_mean(dn_frame[iham,idet,isubset,igain,iscan,0:max_frame-1],3,1)
                        std_col[a5,b,iham,idet,isubset,igain,iscan,1]= $
                          sig_mean(dn_frame[iham,idet,isubset,igain,iscan,0:max_frame-1],3,2)
                        
                        IF dn_col[a5,b,iham,idet,isubset,igain,iscan,1] EQ -999 THEN $
                          dn_col[a5,b,iham,idet,isubset,igain,iscan,1]=0
                        IF std_col[a5,b,iham,idet,isubset,igain,iscan,1] EQ -999 THEN $
                          std_col[a5,b,iham,idet,isubset,igain,iscan,1]=0
                    ENDFOR
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR

;*****************************************************************************
;  Relabel the higher subsample as 0 and the lower as 1.
;*****************************************************************************

    temp1=dblarr(nham,maxdet,maxsubsets,2,max_scan)
    IF a4 EQ 'I' THEN BEGIN
        FOR iham=0, nham-1 DO BEGIN
            FOR idet=0, detectors_TEB[b]-1 DO BEGIN
                FOR igain=0, 1 DO BEGIN
                    FOR iscan=0, max_scan-1 DO BEGIN
                        IF dn_col[a5,b,iham,idet,0,igain,iscan,1] $
                          GT dn_col[a5,b,iham,idet,1,igain,iscan,1] THEN BEGIN
                            temp1[iham,idet,0,igain,iscan]=dn_col[a5,b,iham,idet,0,igain,iscan,1]
                            temp1[iham,idet,1,igain,iscan]=dn_col[a5,b,iham,idet,1,igain,iscan,1]
                        ENDIF ELSE BEGIN
                            temp1[iham,idet,0,igain,iscan]=dn_col[a5,b,iham,idet,1,igain,iscan,1]
                            temp1[iham,idet,1,igain,iscan]=dn_col[a5,b,iham,idet,0,igain,iscan,1]
                        ENDELSE
                    ENDFOR
                ENDFOR
            ENDFOR
        ENDFOR
        dn_col[a5,b,*,*,0,*,*,1]=temp1[*,*,0,*,*]
        dn_col[a5,b,*,*,1,*,*,1]=temp1[*,*,1,*,*]
    ENDIF
ENDFOR

;*****************************************************************************
;  Determine if auto gain or fixed high gain for M13, detector 1 using
;  the first four scans of collect 1.
;*****************************************************************************

IF dn_col[0,3,0,0,0,0,0,1] EQ 0 THEN j1=0 ELSE j1=1
IF dn_col[0,3,0,0,0,0,1,1] EQ 0 THEN j2=0 ELSE j2=1
IF dn_col[0,3,0,0,0,0,2,1] EQ 0 THEN j3=0 ELSE j3=1
IF dn_col[0,3,0,0,0,0,3,1] EQ 0 THEN j4=0 ELSE j4=1

; if all zero then fixed high gain, if one non-zero then suto gain,
; and if two non-zero then fixed low gain.
IF j1+j2+j3+j4 EQ 0 THEN type_gain=0
IF j1+j2+j3+j4 EQ 1 THEN type_gain=1
IF j1+j2+j3+j4 EQ 2 THEN type_gain=2

; Auto Gain: write low background to index with high background and vice versa.
IF type_gain EQ 1 THEN BEGIN
    FOR col=0, ncollects-1 DO BEGIN
        FOR iham=0, nham-1 DO BEGIN
            FOR idet=0, detectors_TEB[b]-1 DO BEGIN
                FOR iscan=0, max_scan-1 DO BEGIN
                    IF iscan+4 LT max_scan-1 THEN BEGIN
                        IF dn_col[col,3,iham,idet,0,0,iscan,1] GT 0 AND dn_col[col,3,iham,idet,0,0,iscan+4,1] GT 0 THEN $
                          dn_col[col,3,iham,idet,0,0,iscan+2,1]=dn_col[col,3,iham,idet,0,0,iscan,1]
                        IF dn_col[col,3,iham,idet,0,1,iscan,1] GT 0 AND dn_col[col,3,iham,idet,0,1,iscan+4,1] GT 0 THEN $
                          dn_col[col,3,iham,idet,0,1,iscan+2,1]=dn_col[col,3,iham,idet,0,1,iscan,1]
                    ENDIF
                    IF iscan LE 1 THEN BEGIN
                        IF dn_col[col,3,iham,idet,0,0,iscan+2,1] GT 0 THEN $
                          dn_col[col,3,iham,idet,0,0,iscan,1]=dn_col[col,3,iham,idet,0,0,iscan+2,1]
                        IF dn_col[col,3,iham,idet,0,1,iscan+2,1] GT 0 THEN $
                          dn_col[col,3,iham,idet,0,1,iscan,1]=dn_col[col,3,iham,idet,0,1,iscan+2,1]
                    ENDIF
                    IF iscan GE max_scan-3 THEN BEGIN
                        IF dn_col[col,3,iham,idet,0,0,iscan-2,1] GT 0 THEN $
                          dn_col[col,3,iham,idet,0,0,iscan,1]=dn_col[col,3,iham,idet,0,0,iscan-2,1]
                        IF dn_col[col,3,iham,idet,0,1,iscan-2,1] GT 0 THEN $
                          dn_col[col,3,iham,idet,0,1,iscan,1]=dn_col[col,3,iham,idet,0,1,iscan-2,1]
                    ENDIF
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR
ENDIF

;*****************************************************************************
;  Read in the dump files for EV.
;*****************************************************************************

q=0
a5=0
FOR ifile=0, nfile-1 DO BEGIN

    nlines=File_Lines(list_files[ifile])
    IF nlines EQ 0 THEN continue
    
    a2=strsplit(list_files[ifile],'/',/Extract)
    a3=N_ELEMENTS(a2)
    a4=strmid(a2[a3-1],strpos(a2[a3-1],'_',/Reverse_Search)+1,1) ; Determine I or M band.
    a8=strmid(a2[a3-1],strpos(a2[a3-1],'/',/Reverse_Search)+10,2) ; Determine EV or SV.

    ; Output file name.
    outfile='/home/jmcintire/VIIRS_Data/RC_5_EDU/'+a2[a3-4]+'/'+a2[a3-3]+'/'+a2[a3-2]+'/' $
      +a2[a3-4]+'.'+a2[a3-3]+'.'+a2[a3-2]+'.dn.dynamic-range.sav'

    ; Keep only the EV.
    IF a8 EQ 'SV' OR a8 EQ 'OB' THEN continue

    ; Determine the relative collect #.
    q=q+1
    IF q EQ 9 THEN BEGIN
        a5=a5+1
        q=1
    ENDIF
    print,list_files[ifile]
    
    read_dmpd2,list_files[ifile],header_size,uaid,collect,scan,ham,apid,band,dnb_s,det,dsect,frame,offset,gain_type,dns

    ; Translate the band name to a number.
    IF band[0] EQ 'I4' THEN b=0
    IF band[0] EQ 'I5' THEN b=1
    IF band[0] EQ 'M12' THEN b=2
    IF band[0] EQ 'M13' THEN b=3
    IF band[0] EQ 'M14' THEN b=4
    IF band[0] EQ 'M15' THEN b=5
    IF band[0] EQ 'M16A' THEN b=6
    IF band[0] EQ 'M16B' THEN b=7
    
    max_frame=max(frame)-min(frame)+1

;*****************************************************************************
;  Write DN values to an array for EV.
;*****************************************************************************

    dn_frame=dblarr(nham,maxdet,maxsubsets,maxgain,max_scan,max_frame)
    iscan=0
    idet=0
    iframe=0
    subset=0
    FOR i=0L, nlines-1 DO BEGIN
        IF scan[i] EQ iscan+1 AND det[i] EQ idet+1 THEN BEGIN
            dn_frame[ham[i],idet,subset,gain_type[i],iscan,iframe]=dns[i]
            iframe=iframe+1
            IF iframe EQ max_frame THEN BEGIN
                idet=idet+1
                iframe=0
                IF idet EQ detectors_TEB[b] THEN BEGIN
                    idet=0
                    iscan=iscan+1
                ENDIF
            ENDIF
        ENDIF
        IF a4 EQ 'I' THEN BEGIN
            IF subset EQ 0 THEN BEGIN
                subset=1
                goto,skip_over_2
            ENDIF
            IF subset EQ 1 THEN subset=0
            skip_over_2:
        ENDIF
    ENDFOR

;*****************************************************************************
;  Subtract the SV average over frames from each frame of EV (keep label of
;  first subsample as 0 and second subsample as 1).
;*****************************************************************************

    dn_col_1=dblarr(nham,maxdet,maxsubsets,maxgain,max_scan,max_frame,2) ; (0=EV, 1=OBC)
    FOR iham=0, nham-1 DO BEGIN
        FOR idet=0, detectors_TEB[b]-1 DO BEGIN
            FOR isubset=0, subsets_TEB[b]-1 DO BEGIN
                FOR igain=0, 1 DO BEGIN
                    FOR iscan=0, max_scan-1 DO BEGIN
                        IF sig_mean(dn_frame[iham,idet,0,igain,iscan,0:max_frame-1],3,1) GT $
                          sig_mean(dn_frame[iham,idet,1,igain,iscan,0:max_frame-1],3,1) THEN flag=0
                        IF sig_mean(dn_frame[iham,idet,0,igain,iscan,0:max_frame-1],3,1) LT $
                          sig_mean(dn_frame[iham,idet,1,igain,iscan,0:max_frame-1],3,1) THEN flag=1
                        IF sig_mean(dn_frame[iham,idet,0,igain,iscan,0:max_frame-1],3,1) EQ -999 AND $
                          sig_mean(dn_frame[iham,idet,1,igain,iscan,0:max_frame-1],3,1) EQ -999 THEN continue
                        FOR iframe=0, max_frame-1 DO BEGIN
                            IF band_type_TEB[b] EQ 1 THEN BEGIN
                                IF dn_frame[iham,idet,isubset,igain,iscan,iframe] NE 0 AND flag EQ 0 THEN BEGIN
                                    
                                    dn_col_1[iham,idet,isubset,igain,iscan,iframe,0]= $
                                      dn_frame[iham,idet,isubset,igain,iscan,iframe] $
                                      -dn_col[a5,b,iham,idet,isubset,igain,iscan,1]
                                    dn_col_1[iham,idet,isubset,igain,iscan,iframe,0]= $
                                      dn_frame[iham,idet,isubset,igain,iscan,iframe] $
                                      -dn_col[a5,b,iham,idet,isubset,igain,iscan,1]
                                ENDIF
                                IF dn_frame[iham,idet,isubset,igain,iscan,iframe] NE 0 AND flag EQ 1 THEN BEGIN
                                    
                                    IF isubset EQ 0 THEN dn_col_1[iham,idet,isubset,igain,iscan,iframe,0]= $
                                      dn_frame[iham,idet,isubset,igain,iscan,iframe] $
                                      -dn_col[a5,b,iham,idet,1,igain,iscan,1]
                                    IF isubset EQ 1 THEN dn_col_1[iham,idet,isubset,igain,iscan,iframe,0]= $
                                      dn_frame[iham,idet,isubset,igain,iscan,iframe] $
                                      -dn_col[a5,b,iham,idet,0,igain,iscan,1]
                                ENDIF
                                IF dn_frame[iham,idet,isubset,igain,iscan,iframe] EQ 0 THEN $
                                  dn_col_1[iham,idet,isubset,igain,iscan,iframe,0]=0
                            ENDIF
                            IF band_type_TEB[b] EQ 0 THEN BEGIN
                                IF dn_frame[iham,idet,isubset,igain,iscan,iframe] NE 0 THEN $
                                  dn_col_1[iham,idet,isubset,igain,iscan,iframe,0]= $
                                  dn_frame[iham,idet,isubset,igain,iscan,iframe] $
                                  -dn_col[a5,b,iham,idet,isubset,igain,iscan,1]
if b eq 3 and iham eq 0 and idet eq 0 and igain eq 0 and iscan eq 97 then print,a5,b,iham,idet,isubset,igain,iscan,iframe,dn_col_1[iham,idet,isubset,igain,iscan,iframe,0],dn_frame[iham,idet,isubset,igain,iscan,iframe],dn_col[a5,b,iham,idet,isubset,igain,iscan,1]
                                IF dn_frame[iham,idet,isubset,igain,iscan,iframe] EQ 0 THEN $
                                  dn_col_1[iham,idet,isubset,igain,iscan,iframe,0]=0
                            ENDIF
                        ENDFOR
                    ENDFOR
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR
        
;*****************************************************************************
;  Average over frames with standard deviation.
;*****************************************************************************

    FOR iham=0, nham-1 DO BEGIN
        FOR idet=0, detectors_TEB[b]-1 DO BEGIN
            FOR isubset=0, subsets_TEB[b]-1 DO BEGIN
                FOR igain=0, 1 DO BEGIN
                    FOR iscan=0, max_scan-1 DO BEGIN
;if b eq 3 and iham eq 0 and idet eq 0 and igain eq 0 and iscan eq 97 then print,a5,b,iham,idet,isubset,igain,iscan,dn_col_1[iham,idet,isubset,igain,iscan,0:max_frame-1,0]
                        dn_col[a5,b,iham,idet,isubset,igain,iscan,0]= $
                          sig_mean(dn_col_1[iham,idet,isubset,igain,iscan,0:max_frame-1,0],3,1)
                        std_col[a5,b,iham,idet,isubset,igain,iscan,0]= $
                          sig_mean(dn_col_1[iham,idet,isubset,igain,iscan,0:max_frame-1,0],3,2)
                        IF dn_col[a5,b,iham,idet,isubset,igain,iscan,0] EQ -999 THEN $
                          dn_col[a5,b,iham,idet,isubset,igain,iscan,0]=0
                        IF std_col[a5,b,iham,idet,isubset,igain,iscan,0] EQ -999 THEN $
                          std_col[a5,b,iham,idet,isubset,igain,iscan,0]=0
if b eq 3 and iham eq 0 and idet eq 0 and igain eq 0 and iscan eq 97 then print,a5,b,iham,idet,isubset,igain,iscan,dn_col[a5,b,iham,idet,isubset,igain,iscan,0]
                    ENDFOR
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR

;*****************************************************************************
;  Average the dn over scans and average the standard deviations.
;*****************************************************************************

    FOR iham=0, nham-1 DO BEGIN
        FOR idet=0, detectors_TEB[b]-1 DO BEGIN
            FOR isubset=0, subsets_TEB[b]-1 DO BEGIN
                FOR igain=0, 1 DO BEGIN
                    dn[a5,b,iham,idet,isubset,igain,0]=sig_mean(dn_col[a5,b,iham,idet,isubset,igain,0:max_scan-1,0],3,1)
if b eq 3 and iham eq 0 and idet eq 0 then print,a5,b,iham,idet,isubset,igain,dn[a5,b,iham,idet,isubset,igain,0],sig_mean(dn_col[a5,b,iham,idet,isubset,igain,0:max_scan-1,0],3,1)
                    x=WHERE(std_col[a5,b,iham,idet,isubset,igain,0:max_scan-1,0] NE 0,num)
                    IF num EQ 1 THEN std[a5,b,iham,idet,isubset,igain,0] $
                      =mean(std_col[a5,b,iham,idet,isubset,igain,0:max_scan-1,0])*2 ELSE $
                      std[a5,b,iham,idet,isubset,igain,0]=sig_mean(std_col[a5,b,iham,idet,isubset,igain,0:max_scan-1,0],3,1)

                    IF dn[a5,b,iham,idet,isubset,igain,0] EQ -999 THEN dn[a5,b,iham,idet,isubset,igain,0]=0
                    IF std[a5,b,iham,idet,isubset,igain,0] EQ -999 THEN std[a5,b,iham,idet,isubset,igain,0]=0
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR
    
;*****************************************************************************
;  Calculate the dn and standard deviation over scans, then average over frames.
;*****************************************************************************

    dn_scan_avg=dblarr(nham,maxdet,maxsubsets,maxgain,max_frame,2) ; (0=EV and 1=OBC)
    std_scan_avg=dblarr(nham,maxdet,maxsubsets,maxgain,max_frame,2)
    dn_avg_over_frames=dblarr(nham,maxdet,maxsubsets,maxgain,2)
    std_avg_over_frames=dblarr(nham,maxdet,maxsubsets,maxgain,2)
    FOR iham=0, nham-1 DO BEGIN
        FOR idet=0, detectors_TEB[b]-1 DO BEGIN
            FOR isubset=0, subsets_TEB[b]-1 DO BEGIN
                FOR igain=0, 1 DO BEGIN
                    FOR iframe=0, max_frame-1 DO BEGIN
                        dn_scan_avg[iham,idet,isubset,igain,iframe,0]= $
                          sig_mean(dn_col_1[iham,idet,isubset,igain,0:max_scan-1,iframe,0],3,1)
                        std_scan_avg[iham,idet,isubset,igain,iframe,0]= $
                          sig_mean(dn_col_1[iham,idet,isubset,igain,0:max_scan-1,iframe,0],3,2)
                        
                        IF dn_scan_avg[iham,idet,isubset,igain,iframe,0] EQ -999 THEN $
                          dn_scan_avg[iham,idet,isubset,igain,iframe,0]=0
                        IF std_scan_avg[iham,idet,isubset,igain,iframe,0] EQ -999 THEN $
                          std_scan_avg[iham,idet,isubset,igain,iframe,0]=0
                    ENDFOR
                    dn_avg_over_frames[iham,idet,isubset,igain,0]= $
                      sig_mean(dn_scan_avg[iham,idet,isubset,igain,0:max_frame-1,0],3,1)
                    std_avg_over_frames[iham,idet,isubset,igain,0]= $
                      sig_mean(std_scan_avg[iham,idet,isubset,igain,0:max_frame-1,0],3,1)
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR
    dn[a5,b,*,*,*,*,1]=dn_avg_over_frames[*,*,*,*,0]
    std[a5,b,*,*,*,*,1]=std_avg_over_frames[*,*,*,*,0]
    dn_col_2[a5,b,*,*,*,*,0:max_frame-1]=dn_scan_avg[*,*,*,*,0:max_frame-1,0]
    std_col_2[a5,b,*,*,*,*,0:max_frame-1]=std_scan_avg[*,*,*,*,0:max_frame-1,0]

;*****************************************************************************
;  Calculate the dn and standard deviation over scans and frames.
;*****************************************************************************

    m=long(max_frame)
    n=long(max_scan)*long(max_frame)
    dn_total=dblarr(nham,maxdet,maxsubsets,maxgain,2) ; (0=EV and 1=OBC)
    std_total=dblarr(nham,maxdet,maxsubsets,maxgain,2)
    FOR iham=0, nham-1 DO BEGIN
        FOR idet=0, detectors_TEB[b]-1 DO BEGIN
            FOR isubset=0, subsets_TEB[b]-1 DO BEGIN
                FOR igain=0, 1 DO BEGIN
                    k=0
                    p=0L
                    temp=dblarr(n/2)
                    temp_obc=dblarr(n/2)
                    FOR i=0L, max_scan-1 DO BEGIN
                        IF dn_col_1[iham,idet,isubset,igain,i,0,0] NE 0 THEN BEGIN
                            temp[k*m:(k+1)*m-1]=dn_col_1[iham,idet,isubset,igain,i,0:max_frame-1,0]
                            k=k+1
                        ENDIF
                    ENDFOR
                    dn_total[iham,idet,isubset,igain,0]=sig_mean(temp[0:n/2-1],3,1)
                    std_total[iham,idet,isubset,igain,0]=sig_mean(temp[0:n/2-1],3,2)
                    
                    IF dn_total[iham,idet,isubset,igain,0] EQ -999 THEN $
                      dn_total[iham,idet,isubset,igain,0]=0
                    IF std_total[iham,idet,isubset,igain,0] EQ -999 THEN $
                      std_total[iham,idet,isubset,igain,0]=0
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR
    dn[a5,b,*,*,*,*,2]=dn_total[*,*,*,*,0]
    std[a5,b,*,*,*,*,2]=std_total[*,*,*,*,0]
ENDFOR

save,/compress,dn,std,dn_col,std_col,dn_col_2,std_col_2,filename=outfile

skipout:

END

