;*****************************************************************************
;				NICST TOOL
;
;  Uses a quadratic polynominal fit in the dynamic range and a straight line fit
;  above the dynamic range. Then it calculates the intersection point.
;
;  Input:
;  1) Input file.
;
;  Output:
;  1) saturation_dn[band,HAM,detector,subsample]
;  1) saturation_L[band,HAM,detector,subsample]
;  1) saturation_temp[band,HAM,detector,subsample]
;  2) saturation_dn_band[band]
;  3) saturation_L_band[band]
;  4) saturation_temp_band[band]
;  
;  v1.0 - 07/17/2008 - Jeff McIntire
;*****************************************************************************

PRO RC5_dynamic_range, infile

;*****************************************************************************
;  Read in input file, sav TEB file, and make some definitions.
;*****************************************************************************

nlines=File_Lines(infile)
ab=''
header=strarr(1)
inpath=strarr(1)		; Input path.
outpath=strarr(1)		; Output path.
rvs_in=strarr(1)
param_in=strarr(1)
rsrpath=strarr(1)
fnumber=fltarr(1)

openr,lun,infile,/Get_Lun	
readf,lun,header	
readf,lun,inpath
readf,lun,outpath
readf,lun,rvs_in
readf,lun,param_in
readf,lun,rsrpath
readf,lun,fnumber
Free_Lun,lun

a2=strsplit(outpath,'/',/Extract)
a3=N_ELEMENTS(a2)

infname1=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.dn.dynamic-range.sav'
infname2=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rad.sav'
infname3=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.temp.sav'
restore,infname1
restore,infname2
restore,infname3

restore,'/home/jmcintire/VIIRS_Data/general.TEB.sav'

outfile=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.dyn.sav'

x=dblarr(ncollects)
z=dblarr(ncollects)

rg_1=dblarr(nbands_TEB,nham,maxdet,maxsubsets,2)
rg_2=dblarr(nbands_TEB,nham,maxdet,maxsubsets,2)
saturation_dn=dblarr(nbands_TEB,nham,maxdet,maxsubsets)
saturation_L=dblarr(nbands_TEB,nham,maxdet,maxsubsets)
saturation_temp=dblarr(nbands_TEB,nham,maxdet,maxsubsets)
saturation_dn_band=dblarr(nbands_TEB)
saturation_L_band=dblarr(nbands_TEB)
saturation_temp_band=dblarr(nbands_TEB)

;*****************************************************************************
;  Fit using Quadratic POLY_FIT in the dynamic range.
;*****************************************************************************

FOR b=0, nbands_TEB-1 DO BEGIN
	FOR is=0, subsets_TEB[b]-1 DO BEGIN
		FOR ih=0, nham-1 DO BEGIN
			FOR id=0, detectors_TEB[b]-1 DO BEGIN
				lcol=0
				hcol=ncollects
				FOR col=0, ncollects-1 DO BEGIN
					IF rad_TMC[col,b,ih,id,is,0]*0.4444 LT L_min_TEB[b,0] THEN lcol=lcol+1
					IF rad_TMC[col,b,ih,id,is,0]*0.4444 GT L_max_TEB[b,0] THEN hcol=hcol-1
				ENDFOR
if b eq 3 then hcol=hcol+1
				x=dn[lcol:hcol-1,b,ih,id,is,1,0]
				z=rad_TMC[lcol:hcol-1,b,ih,id,is,0]*0.4444

				result=POLY_FIT(z,x,1)
				rg_1[b,ih,id,is,0:1]=result[0:1]

;*****************************************************************************
;  Make a straight line fit above the dynamic range.
;*****************************************************************************

				x=dn[hcol:ncollects-1,b,ih,id,is,1,0]
				z=rad_TMC[hcol:ncollects-1,b,ih,id,is,0]*0.4444

				result1=POLY_FIT(z,x,1)
				rg_2[b,ih,id,is,0:1]=result1[0:1]

if id eq 0 and ih eq 0 and is eq 0 then print,b,ih,id,is,dn[hcol:ncollects-1,b,ih,id,is,1,0]
if id eq 0 and ih eq 0 and is eq 0 then print,b,ih,id,is,rad_TMC[hcol:ncollects-1,b,ih,id,is,0]*0.4444
;*****************************************************************************
;  Use the quadratic forumla to determine the intersept between these two lines.
;*****************************************************************************

				saturation_L[b,ih,id,is]=(rg_1[b,ih,id,is,0]-rg_2[b,ih,id,is,0]) $
					/(rg_2[b,ih,id,is,1]-rg_1[b,ih,id,is,1])
				saturation_dn[b,ih,id,is]=rg_2[b,ih,id,is,0]+rg_2[b,ih,id,is,1]*saturation_L[b,ih,id,is]

				c2=14387.86D				; h*c/k*10^6
				c1=119106211.8D				; 2*h*c^2*10^(6*5)

				saturation_temp[b,ih,id,is]=c2/lambda_TEB[b]/alog(c1/lambda_TEB[b]^5/saturation_L[b,ih,id,is]+1.)
			ENDFOR
		ENDFOR
	ENDFOR
	saturation_dn_band[b]=mean(saturation_dn[b,0:1,0:detectors_TEB[b]-1,0:subsets_TEB[b]-1])
	saturation_L_band[b]=mean(saturation_L[b,0:1,0:detectors_TEB[b]-1,0:subsets_TEB[b]-1])
	saturation_temp_band[b]=mean(saturation_temp[b,0:1,0:detectors_TEB[b]-1,0:subsets_TEB[b]-1])
print,b,saturation_dn_band[b],saturation_L_band[b],saturation_temp_band[b]
ENDFOR

save,/compress,saturation_dn,saturation_L,saturation_temp,saturation_dn_band,saturation_L_band, $
	saturation_temp_band,filename=outfile

;*****************************************************************************
;  Plot the dynamic range and fits.
;*****************************************************************************

set_multicolor,xsave,ysave,psave,chname,lcol,symbol,red,green,blue
set_plot,'ps'
outfname=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.dynamic_range.ps'
device,/landscape,filename=outfname,/color

!p.multi=[0,1,1]
!p.position=0

D=findgen(5000)
FOR b=0, nbands_TEB-1 DO BEGIN
	lcollect=0
	hcollect=ncollects
	FOR col=0, ncollects-1 DO BEGIN
		IF rad_TMC[col,b,0,1,0,0]*0.4444 LT L_min_TEB[b,0] THEN lcollect=lcollect+1
		IF rad_TMC[col,b,0,1,0,0]*0.4444 GT L_max_TEB[b,0] THEN hcollect=hcollect-1
	ENDFOR

	title1='Dynamic Range Requirement (SRV0466)'
	title2=band_name_TEB[b]+', HAM=A'

	ymax=max(dn[*,b,0,1,0,1,0])
	xmax=max(rad_TMC[*,b,0,1,0,0]*0.4444)

	plot,dn[*,b,0,1,0,1,0],rad_TMC[*,b,0,1,0,0]*0.4444,yrange=[0,ymax],xrange=[0,xmax], $
		xtit='Radiance',ytit='dn',title=title2,charsize=1.2,charthick=5,/nodata
	FOR id=0, detectors_TEB[b]-1 DO BEGIN
		oplot,rad_TMC[lcollect:hcollect-1,b,0,id,0,0]*0.4444,dn[lcollect:hcollect-1,b,0,id,0,1,0], $
			symsize=1,color=lcol[id],psym=symbol[id]
		oplot,rg_1[b,0,id,0,0]+rg_1[b,0,id,0,1]*D[*],color=lcol[id]
		oplot,rad_TMC[hcollect:ncollects-1,b,0,id,0,0]*0.4444,dn[hcollect:ncollects-1,b,0,id,0,1,0], $
			symsize=1,color=lcol[id],psym=symbol[id]
		oplot,rg_2[b,0,id,0,0]+rg_2[b,0,id,0,1]*D[*],color=lcol[id]
	ENDFOR
	oplot,[L_max_TEB[b,0],L_max_TEB[b,0]],[-100,10000],symsize=1,color=lcol[1],linestyle=2,thick=2
	XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=1.6,charthick=5
ENDFOR

device,/close

END
