;*****************************************************************************
;				NICST TOOL
;
;  EDU RC-5 Part 1 test for TEB bands. Read the temperature data extracted using the LRV and
;  reformat it and put it in the proper units. Read the temperature data from the
;  GSE files and reformat. Calculate the slopes of the temperatures over scan
;  and collect. Then write to a sav file. This routine is for band M13 only from 
;  the tests using the TMC.
;  
;  Input:
;    1) Directory with LRV extracted files and GSE files.
;    2) Initial HAM side for each collect.
;
;  Output:
;    1) Temp of BCS, SVS, TMC, OMM, LWASP, MWASP, and CFPA as a function of collects
;        (labeled T_xxx[collects]).
;    2) Temp of RTA, CAV, SH, OBC, and HAM as a function of collects and HAM side
;        (labeled T_xxx[HAM,collects]).
;    3) Coefficients of straight line fit of temp versus scans: BCS and SVS 
;        (labeled Slope_temp_5[i,coeff]); and OMM, LWASP, MWASP, and CFPA 
;        (labeled Slope_temp_2[i,collects,coeff]).
;    4) Coefficients of straight line fit of temp versus collects: BCS and SVS 
;        (labeled Slope_temp_6[i,coeff]); and OMM, LWASP, MWASP, and CFPA 
;        (labeled Slope_temp_4[i,coeff]).
;    5) Coefficients of straight line fit of temp versus scans:
;	 RTA, CAV, SH, OBC, and HAM (labeled Slope_temp_1[i,collects,HAM,coeff]).
;    6) Coefficients of straight line fit of temp versus collects: 
;        RTA, CAV, SH, OBC, and HAM (labeled Slope_temp_3[i,HAM,coeff]).
;
;  v1.0 - 03/31/2008 - Jeff McIntire
;*****************************************************************************

PRO RC5_Gain_read_LRV, infile

;*****************************************************************************
;  Read in input file.
;*****************************************************************************

nlines=File_Lines(infile)
ab=''
header=strarr(1)
inpath=strarr(1)
directory=strarr(1)			; Input path.
rvs_in=strarr(1)
param_in=strarr(1)
rsrpath=strarr(1)
fnumber=strarr(1)
header_size=intarr(1)

openr,lun,infile,/Get_Lun	
readf,lun,header	
readf,lun,inpath
readf,lun,directory
readf,lun,rvs_in
readf,lun,param_in
readf,lun,rsrpath
readf,lun,fnumber
readf,lun,header_size
Free_Lun,lun

n_collect,inpath,ncollects,type,0
ham=initial_ham_side(inpath,header_size)

;*****************************************************************************
;  Some definitions.
;*****************************************************************************

nscans=100
nham=2
DT_rta=4

a2=strsplit(directory,'/',/Extract)
a3=N_ELEMENTS(a2)

outfile=directory+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.'

;*****************************************************************************
;  Read in the LRV extracted temperatures.
;*****************************************************************************

; Get list of .txt files from specified path. 
spawn, 'ls -C1 '+inpath+'LRV/*.txt',list_files

IF N_ELEMENTS(list_files) EQ 0 THEN goto,skipout
nfile=N_ELEMENTS(list_files)

; Temperature definitions.
T_rta_1=dblarr(2,nscans,nham,nfile)
T_rta_2=dblarr(nscans,nham,nfile)
T_rta=dblarr(nham,nfile)	
T_cav_2=dblarr(nscans,nham,nfile)
T_cav=dblarr(nham,nfile)	
T_sh_1=dblarr(2,nscans,nham,nfile)
T_sh_2=dblarr(nscans,nham,nfile)
T_sh=dblarr(nham,nfile)
T_ham_1=dblarr(2,nscans,nham,nfile)
T_ham_2=dblarr(nscans,nham,nfile)
T_ham=dblarr(nham,nfile)
T_obc_1=dblarr(6,nscans,nham,nfile)
T_obc_2=dblarr(nscans,nham,nfile)
T_obc=dblarr(nham,nfile)
T_cfpa_1=dblarr(2,nscans,nfile)
T_cfpa_2=dblarr(nscans,nfile)
T_cfpa=dblarr(nfile)
T_omm_1=dblarr(2,nscans,nfile)
T_omm_2=dblarr(nscans,nfile)
T_omm=dblarr(nfile)
T_lwasp_1=dblarr(nscans,nfile)
T_lwasp=dblarr(nfile)
T_mwasp_1=dblarr(nscans,nfile)
T_mwasp=dblarr(nfile)

; Definitions used in drift calulation.
f=findgen(nscans)
h=findgen(nscans/2)
g=findgen(nfile)
z=dblarr(nfile)
slope_temp_1=dblarr(5,nfile,nham,2)
slope_temp_2=dblarr(4,nfile,2)
slope_temp_3=dblarr(5,nham,2)
slope_temp_4=dblarr(4,2)
slope_temp_5=dblarr(2,nfile,2)
slope_temp_6=dblarr(2,2)

FOR j=0, nfile-1 DO BEGIN

	nlines=File_Lines(list_files[j])
	IF nlines EQ 0 THEN goto,skipfile

	awa=''
	header1=strarr(1)
	header2=strarr(1)
	scan=intarr(nlines)		; Scan number.
	LRV=intarr(nlines)		; LRV number.
	EngUnits=fltarr(nlines)		; Number extracted.

	openr,lun,list_files[j],/Get_Lun	
	readf,lun,header1
	readf,lun,header2

	FOR ix=0L, nlines-3 DO BEGIN
		readf,lun,awa
		a1=strsplit(awa,String(9B),/Extract)
		scan(ix)=a1(3)
		LRV(ix)=a1(4)
		EngUnits(ix)=DOUBLE(a1(7))
	ENDFOR
	Free_Lun,lun

;*****************************************************************************
;  Convert Celcius to Kelvin.
;*****************************************************************************

	FOR ix=0L, nlines-3 DO BEGIN
		IF LRV[ix] EQ 29024 THEN EngUnits[ix]=EngUnits[ix]+273.15
		IF LRV[ix] EQ 21028 THEN EngUnits[ix]=EngUnits[ix]+273.15
		IF LRV[ix] EQ 21029 THEN EngUnits[ix]=EngUnits[ix]+273.15
		IF LRV[ix] EQ 29017 THEN EngUnits[ix]=EngUnits[ix]+273.15
		IF LRV[ix] EQ 29016 THEN EngUnits[ix]=EngUnits[ix]+273.15
		IF LRV[ix] EQ 29014 THEN EngUnits[ix]=EngUnits[ix]+273.15
		IF LRV[ix] EQ 29011 THEN EngUnits[ix]=EngUnits[ix]+273.15
		IF LRV[ix] EQ 12021 THEN EngUnits[ix]=EngUnits[ix]+273.15
		IF LRV[ix] EQ 12022 THEN EngUnits[ix]=EngUnits[ix]+273.15
		IF LRV[ix] EQ 29005 THEN EngUnits[ix]=EngUnits[ix]+273.15
	ENDFOR

;*****************************************************************************
;  Write the temperatures to useful arrays.
;*****************************************************************************

	FOR ix=0L, nlines-3 DO BEGIN
		IF ix GT 0 THEN BEGIN 
			IF scan[ix] GT scan[ix-1] AND ham[j] EQ 0 THEN BEGIN
				ham[j]=1
				goto,skip
			ENDIF
			IF scan[ix] GT scan[ix-1] AND ham[j] EQ 1 THEN ham[j]=0
			skip:
		ENDIF

		IF LRV[ix] EQ 29011 THEN T_rta_1[0,scan[ix]-1,ham[j],j]=EngUnits[ix]
		IF LRV[ix] EQ 29014 THEN T_rta_1[1,scan[ix]-1,ham[j],j]=EngUnits[ix]
		IF LRV[ix] EQ 29016 THEN T_sh_1[0,scan[ix]-1,ham[j],j]=EngUnits[ix]
		IF LRV[ix] EQ 29017 THEN T_sh_1[1,scan[ix]-1,ham[j],j]=EngUnits[ix]
		IF LRV[ix] EQ 12021 THEN T_ham_1[0,scan[ix]-1,ham[j],j]=EngUnits[ix]
		IF LRV[ix] EQ 12022 THEN T_ham_1[1,scan[ix]-1,ham[j],j]=EngUnits[ix]
		IF LRV[ix] EQ 2001 THEN T_obc_1[0,scan[ix]-1,ham[j],j]=EngUnits[ix]
		IF LRV[ix] EQ 22002 THEN T_obc_1[1,scan[ix]-1,ham[j],j]=EngUnits[ix]
		IF LRV[ix] EQ 2003 THEN T_obc_1[2,scan[ix]-1,ham[j],j]=EngUnits[ix]
		IF LRV[ix] EQ 2004 THEN T_obc_1[3,scan[ix]-1,ham[j],j]=EngUnits[ix]
		IF LRV[ix] EQ 22005 THEN T_obc_1[4,scan[ix]-1,ham[j],j]=EngUnits[ix]
		IF LRV[ix] EQ 2006 THEN T_obc_1[5,scan[ix]-1,ham[j],j]=EngUnits[ix]
		IF LRV[ix] EQ 27007 THEN T_cfpa_1[0,scan[ix]-1,j]=EngUnits[ix]
		IF LRV[ix] EQ 27009 THEN T_cfpa_1[1,scan[ix]-1,j]=EngUnits[ix]
		IF LRV[ix] EQ 29024 THEN T_omm_1[0,scan[ix]-1,j]=EngUnits[ix]
		IF LRV[ix] EQ 29005 THEN T_omm_1[1,scan[ix]-1,j]=EngUnits[ix]
		IF LRV[ix] EQ 21028 THEN T_lwasp_1[scan[ix]-1,j]=EngUnits[ix]
		IF LRV[ix] EQ 21029 THEN T_mwasp_1[scan[ix]-1,j]=EngUnits[ix]
	ENDFOR

;*****************************************************************************
;  Average the temperatures over blocks and scans. Also, determine the drifting
;  over scans for each temperature.
;*****************************************************************************

	y=dblarr(nscans)
	y1=dblarr(nscans/2)

	T_rta_2[*,*,j]=(T_rta_1[0,*,*,j]+T_rta_1[1,*,*,j])/2
	FOR iham=0, nham-1 DO BEGIN
		T_rta[iham,j]=sig_mean(T_rta_2[*,iham,j],3,1)+DT_rta

		m=0
		FOR k=0, nscans-1 DO BEGIN
			IF T_rta_2[k,iham,j] NE 0 THEN BEGIN
				y1[m]=T_rta_2[k,iham,j]
				m=m+1
			ENDIF
		ENDFOR
		result1=poly_fit(h,y1,1)
		slope_temp_1[0,j,iham,*]=result1[*]
	ENDFOR

	T_cav_2[*,*,j]=(T_rta_1[0,*,*,j]+T_rta_1[1,*,*,j])/2
	FOR iham=0, nham-1 DO BEGIN
		T_cav[iham,j]=sig_mean(T_cav_2[*,iham,j],3,1)

		m=0
		FOR k=0, nscans-1 DO BEGIN
			IF T_cav_2[k,iham,j] NE 0 THEN BEGIN
				y1[m]=T_cav_2[k,iham,j]
				m=m+1
			ENDIF
		ENDFOR
		result1=poly_fit(h,y1,1)
		slope_temp_1[1,j,iham,*]=result1[*]
	ENDFOR

	T_sh_2[*,*,j]=(T_sh_1[0,*,*,j]+T_sh_1[1,*,*,j])/2
	FOR iham=0, nham-1 DO BEGIN
		T_sh[iham,j]=sig_mean(T_sh_2[*,iham,j],3,1)

		m=0
		FOR k=0, nscans-1 DO BEGIN
			IF T_sh_2[k,iham,j] NE 0 THEN BEGIN
				y1[m]=T_sh_2[k,iham,j]
				m=m+1
			ENDIF
		ENDFOR
		y[*]=T_sh_2[*,iham,j]
		result1=poly_fit(h,y1,1)
		slope_temp_1[2,j,iham,*]=result1[*]
	ENDFOR

	T_ham_2[*,*,j]=(T_ham_1[0,*,*,j]+T_ham_1[1,*,*,j])/2
	FOR iham=0, nham-1 DO BEGIN
		T_ham[iham,j]=sig_mean(T_ham_2[*,iham,j],3,1)

		m=0
		FOR k=0, nscans-1 DO BEGIN
			IF T_ham_2[k,iham,j] NE 0 THEN BEGIN
				y1[m]=T_ham_2[k,iham,j]
				m=m+1
			ENDIF
		ENDFOR
		result1=poly_fit(h,y1,1)
		slope_temp_1[3,j,iham,*]=result1[*]
	ENDFOR

	T_obc_2[*,*,j]=(T_obc_1[0,*,*,j]+T_obc_1[0,*,*,j]+T_obc_1[2,*,*,j]+T_obc_1[3,*,*,j]+ $
		T_obc_1[4,*,*,j]+T_obc_1[5,*,*,j])/6
	FOR iham=0, nham-1 DO BEGIN
		T_obc[iham,j]=sig_mean(T_obc_2[*,iham,j],3,1)

		m=0
		FOR k=0, nscans-1 DO BEGIN
			IF T_obc_2[k,iham,j] NE 0 THEN BEGIN
				y1[m]=T_obc_2[k,iham,j]
				m=m+1
			ENDIF
		ENDFOR
		result1=poly_fit(h,y1,1)
		slope_temp_1[4,j,iham,*]=result1[*]
	ENDFOR

	T_cfpa_2[*,j]=(T_cfpa_1[0,*,j]+T_cfpa_1[1,*,j])/2
	T_cfpa[j]=sig_mean(T_cfpa_2[*,j],3,1)
	y[*]=T_cfpa_2[*,j]
	result1=poly_fit(f,y,1)
	slope_temp_2[0,j,*]=result1[*]

	T_omm_2[*,j]=(T_omm_1[0,*,j]+T_omm_1[1,*,j])/2
	T_omm[j]=sig_mean(T_omm_2[*,j],3,1)
	y[*]=T_omm_2[*,j]
	result1=poly_fit(f,y,1)
	slope_temp_2[1,j,*]=result1[*]

	T_lwasp[j]=sig_mean(T_lwasp_1[*,j],3,1)
	y[*]=T_lwasp_1[*,j]
	result1=poly_fit(f,y,1)
	slope_temp_2[2,j,*]=result1[*]

	T_mwasp[j]=sig_mean(T_mwasp_1[*,j],3,1)
	y[*]=T_mwasp_1[*,j]
	result1=poly_fit(f,y,1)
	slope_temp_2[3,j,*]=result1[*]

	FOR k=0, 4 DO BEGIN
		FOR iham=0, nham-1 DO BEGIN
			IF slope_temp_1[k,j,iham,1] GT 0.001 THEN print,j,iham,k,slope_temp_1[k,j,iham,1]
		ENDFOR
	ENDFOR

	FOR k=0, 3 DO BEGIN
		IF slope_temp_2[k,j,1] GT 0.001 THEN print,j,k,slope_temp_2[k,j,1]
	ENDFOR

	skipfile:
ENDFOR

;*****************************************************************************
;  Determine the drifting over collects for each temperature.
;*****************************************************************************

FOR iham=0, nham-1 DO BEGIN
	z[*]=T_rta[iham,*]
	result2=poly_fit(g,z,1)
	slope_temp_3[0,iham,*]=result2[*]

	z[*]=T_cav[iham,*]
	result2=poly_fit(g,z,1)
	slope_temp_3[1,iham,*]=result2[*]

	z[*]=T_sh[iham,*]
	result2=poly_fit(g,z,1)
	slope_temp_3[2,iham,*]=result2[*]

	z[*]=T_ham[iham,*]
	result2=poly_fit(g,z,1)
	slope_temp_3[3,iham,*]=result2[*]

	z[*]=T_obc[iham,*]
	result2=poly_fit(g,z,1)
	slope_temp_3[4,iham,*]=result2[*]
ENDFOR

z[*]=T_cfpa[*]
result2=poly_fit(g,z,1)
slope_temp_4[0,*]=result2[*]

z[*]=T_omm[*]
result2=poly_fit(g,z,1)
slope_temp_4[1,*]=result2[*]

z[*]=T_lwasp[*]
result2=poly_fit(g,z,1)
slope_temp_4[2,*]=result2[*]

z[*]=T_mwasp[*]
result2=poly_fit(g,z,1)
slope_temp_4[3,*]=result2[*]

FOR k=0, 4 DO BEGIN
	FOR iham=0, nham-1 DO BEGIN
		IF slope_temp_3[k,iham,1] GT 0.1 THEN print,iham,k,slope_temp_3[k,iham,1]
	ENDFOR
ENDFOR

FOR k=0, 3 DO BEGIN
	IF slope_temp_4[k,1] GT 0.1 THEN print,k,slope_temp_4[k,1]
ENDFOR

;*****************************************************************************
;  Read in the GSE extracted temperatures.
;*****************************************************************************

; Get list of .txt files from specified path. 
spawn, 'ls -C1 '+inpath+'GSE/*.gse',list_files

IF N_ELEMENTS(list_files) EQ 0 THEN goto,skipout_1
nfile=N_ELEMENTS(list_files)

; Definitions.
T_bcs_1=dblarr(nscans,nfile)
T_svs_1=dblarr(nscans,nfile)
T_bcs=dblarr(nfile)
T_svs=dblarr(nfile)

FOR j=0, nfile-1 DO BEGIN

	nlines=File_Lines(list_files[j])
	IF nlines EQ 0 THEN goto,skipfile_1

	awa=''
	name=strarr(nlines)		; Name of quantity.
	temp=dblarr(nlines)		; Temperature.

	openr,lun,list_files[j],/Get_Lun

	FOR ix=0L, nlines-1 DO BEGIN
		readf,lun,awa
		a1=strsplit(awa,String(9B),/Extract)
		name(ix)=a1(1)
		temp(ix)=DOUBLE(a1(5))
	ENDFOR
	Free_Lun,lun

;*****************************************************************************
;  Convert Celcius to Kelvin.
;*****************************************************************************

	FOR ix=0L, nlines-1 DO BEGIN
		IF name[ix] EQ 'BS_BCS_EFT_TMP' THEN temp[ix]=temp[ix]+273.15
		IF name[ix] EQ 'BS_SVS_EFT_TMP' THEN temp[ix]=temp[ix]+273.15
	ENDFOR

;*****************************************************************************
;  Write the temperatures to useful arrays.
;*****************************************************************************

	k1=0
	k2=0
	FOR ix=0L, nlines-1 DO BEGIN
		IF name[ix] EQ 'BS_BCS_EFT_TMP' THEN BEGIN
			T_bcs_1[k1,j]=temp[ix]
			k1=k1+1
		ENDIF
		IF name[ix] EQ 'BS_SVS_EFT_TMP' THEN BEGIN
			T_svs_1[k2,j]=temp[ix]
			k2=k2+1
		ENDIF
	ENDFOR

;*****************************************************************************
;  Average over available data and detemine drifting over available scans.
;*****************************************************************************

	T_bcs[j]=sig_mean(T_bcs_1[*,j],3,1)
	T_svs[j]=sig_mean(T_svs_1[*,j],3,1)

	slope_temp_5[0,1]=(T_bcs_1[k1-1,j]-T_bcs_1[0,j])/100
	slope_temp_5[1,1]=(T_svs_1[k2-1,j]-T_svs_1[0,j])/100

	FOR k=0, 1 DO BEGIN
		IF slope_temp_5[k,j,1] GT 0.01 THEN print,j,k,slope_temp_5[k,j,1]
	ENDFOR

	skipfile_1:
ENDFOR

;*****************************************************************************
;  Detemine drifting over collects and save to a file.
;*****************************************************************************

f=findgen(nfile)
y=dblarr(nfile)
y[*]=T_bcs[*]
result4=poly_fit(f,y,1)
slope_temp_6[0,*]=result4[*]

f=findgen(nfile)
y=dblarr(nfile)
y[*]=T_svs[*]
result4=poly_fit(f,y,1)
slope_temp_6[1,*]=result4[*]

FOR k=0, 1 DO BEGIN
	IF slope_temp_6[k,1] GT 0.1 THEN print,k,slope_temp_6[k,1]
ENDFOR

;*****************************************************************************
;  Read in the TMC temperatures from the adf files.
;*****************************************************************************

; Get list of .txt files from specified path. 
spawn, 'ls -C1 '+inpath+'ADF/*.adf',list_files

IF N_ELEMENTS(list_files) EQ 0 THEN goto,skipout_1
nfile=N_ELEMENTS(list_files)

; Definitions.
T_tmc=dblarr(nfile)

FOR j=0, nfile-1 DO BEGIN

	nlines=File_Lines(list_files[j])
	IF nlines EQ 0 THEN goto,skipfile_1

	awa=''
	header1=strarr(1)
	header2=strarr(1)
	name=strarr(1)

	openr,lun,list_files[j],/Get_Lun
	readf,lun,header1
	readf,lun,header2
	readf,lun,name

	T_tmc[j]=strmid(name[0],strpos(name[0],'=',/Reverse_Search)+2,5)+273.15

	Free_Lun,lun
ENDFOR
print,T_tmc

save,/compress,T_bcs,T_svs,T_mwasp,T_lwasp,T_omm,T_cfpa,T_obc,T_sh,T_cav,T_rta,T_ham,T_tmc, $
	slope_temp_1,slope_temp_2,slope_temp_3,slope_temp_4,slope_temp_5,slope_temp_6,filename=outfile+'temp.sav'

skipout:
skipout_1:

END
