;*****************************************************************************
;				NICST TOOL
;
;  Determines the gain switching for M13.
;
;  Input:
;  1) Input file.
;
;  Output:
;  1) .ps file with plots.
;  
;  v1.0 - 08/27/2008 - Jeff McIntire
;*****************************************************************************

PRO RC5_Gain_switch, infile

;*****************************************************************************
;  Read in input file, sav TEB file, and make some definitions.
;*****************************************************************************

nlines=File_Lines(infile)
ab=''
header=strarr(1)
inpath=strarr(1)		; Input path.
outpath=strarr(1)		; Output path.
rvs_in=strarr(1)
param_in=strarr(1)
rsrpath=strarr(1)
fnumber=fltarr(1)

openr,lun,infile,/Get_Lun	
readf,lun,header	
readf,lun,inpath
readf,lun,outpath
readf,lun,rvs_in
readf,lun,param_in
readf,lun,rsrpath
readf,lun,fnumber
Free_Lun,lun

a2=strsplit(outpath,'/',/Extract)
a3=N_ELEMENTS(a2)

infname1=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.dn.dynamic-range.sav'
infname2=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rad.sav'
infname3=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.temp.sav'
restore,infname1
restore,infname2
restore,infname3

restore,'/home/jmcintire/VIIRS_Data/general.TEB.sav'

;*****************************************************************************
;  Put the dn in one array for both high and low gain.
;*****************************************************************************

dn_x=fltarr(ncollects,nham,detectors_TEB[3])
rad_x=fltarr(ncollects,nham,detectors_TEB[3])
FOR collect=0, ncollects-1 DO BEGIN
    FOR iham=0, nham-1 DO BEGIN
        FOR idet=0, detectors_TEB[3]-1 DO BEGIN
            IF dn[collect,3,iham,idet,0,1,0] NE 0 THEN dn_x[collect,iham,idet]=dn[collect,3,iham,idet,0,1,0]
            IF dn[collect,3,iham,idet,0,0,0] NE 0 THEN dn_x[collect,iham,idet]=dn[collect,3,iham,idet,0,0,0]
            rad_x[collect,iham,idet]=rad_TMC[collect,3,iham,idet,0,0]
        ENDFOR
    ENDFOR
ENDFOR

;*****************************************************************************
;  Plot the gain switching.
;*****************************************************************************

set_multicolor,xsave,ysave,psave,chname,lcol,symbol,red,green,blue
set_plot,'ps'
outfname=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.gain_switching.ps'
device,/landscape,filename=outfname,/color

!p.multi=[0,1,1]
!p.position=0

D=findgen(5000)

; Gain switching vs radiance.
title1='Gain Switching Requirement (SRV0468)'

ymax=max(dn_x[*,*,*])
xmax=max(rad_x[*,*,*])

FOR ih=0, 1 DO BEGIN
    title2=band_name_TEB[3]+', HAM='+strtrim(string(ih+1),2)

    plot,rad_x[*,ih,0],dn_x[*,ih,0],yrange=[0,ymax],xrange=[0,xmax],xtit='Radiance',ytit='dn',title=title2,charsize=1.2,charthick=5,/nodata
    FOR id=0, detectors_TEB[3]-1 DO BEGIN
        oplot,rad_x[0:ncollects-1,ih,id],dn_x[0:ncollects-1,ih,id],symsize=1,color=lcol[id],psym=symbol[id]
        oplot,rad_x[0:ncollects-1,ih,id],dn_x[0:ncollects-1,ih,id],color=lcol[id]
    ENDFOR
    oplot,[L_max_TEB[3,0],L_max_TEB[3,0]],[-100,10000],symsize=1,color=lcol[1],linestyle=2,thick=2
    XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=1.6,charthick=5
    set_legend,1,chname,lcol,symbol
ENDFOR

; Gain switching vs temperature
xmax=max(T_tmc[*])

FOR ih=0, 1 DO BEGIN
    title2=band_name_TEB[3]+', HAM='+strtrim(string(ih+1),2)

    plot,T_tmc[0:ncollects-1],dn_x[*,ih,0],yrange=[0,ymax],xrange=[0,xmax],xtit='T_tmc (K)',ytit='dn',title=title2,charsize=1.2,charthick=5,/nodata
    FOR id=0, detectors_TEB[3]-1 DO BEGIN
        oplot,T_tmc[0:ncollects-1],dn_x[0:ncollects-1,ih,id],symsize=1,color=lcol[id],psym=symbol[id]
        oplot,T_tmc[0:ncollects-1],dn_x[0:ncollects-1,ih,id],color=lcol[id]
    ENDFOR
    oplot,[L_max_TEB[3,0],L_max_TEB[3,0]],[-100,10000],symsize=1,color=lcol[1],linestyle=2,thick=2
    XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=1.6,charthick=5
    set_legend,1,chname,lcol,symbol
ENDFOR

device,/close

END
